#include "convert.h"


/****       *  HELP  *       ****/

#define	IN_FILE_ERR "  infile <%s> not found!\n"
#define	OUT_FILE_ERR "  cannot open outfile <%s>!\n"
#define	UTF16_NO_DETECT_ERR "  can't detect utf-16 be/le version\n"
#define	END_LINE_ERR "  define out_end_line_type but not define in_end_line_type\n"
#define	FILE_ERR "  ERROR: infile = outfile\n"

void print_help()
{
	printf("Convert Unicode (UTF) text file\n\n");
	printf("usage: utf_convert in_type in_end_line_type out_type out_end_line_type [infile] [outfile]\n");
	printf(" if infile / outfile is not declared program used standard input / output\n");
	printf("\nConvert Unicode from (in_type - description):\n");
	printf(" utf7     -   UTF-7\n");
	printf(" utf8     -   UTF-8\n");
	printf(" utf16    -   UTF-16 (UCS-2) autodetect Big Endian (high endian) or Little Endian (low endian)\n");
	printf(" utf16be  -   UTF-16 (UCS-2) - Big Endian (high endian)\n");
	printf(" utf16le  -   UTF-16 (UCS-2) - Little Endian (low endian)\n");
	printf("\nConvert Unicode to (out_type - description):\n");
	printf(" utf7     -   UTF-7\n");
	printf(" utf8     -   UTF-8\n");
	printf(" utf16be  -   UTF-16 (UCS-2) - Big Endian (high endian)\n");
	printf(" utf16le  -   UTF-16 (UCS-2) - Little Endian (low endian)\n");
	printf(" -        -   not convert Unicode\n");
	printf("\nConvert end line style (in_end_line_type=out_end_line_type - description):\n");
	printf(" lf       -   UNIX\n");
	printf(" crlf     -   DOS / WINDOWS\n");
	printf(" cr       -   MAC\n");
	printf(" -        -   not convert end line\n");
	printf("\n* UTF-7 with optional characters encoded directly actually is not suppored :-(");
	printf("\n* UCS-2 for Java actually is not suppored :-(");
	printf("\n* UTF-32 (UCS-4) actually is not suppored :-(\n");
}



/****       *  MAIN  *       ****/

int main(int argc, char *argv[])
{
	FILE	*ifp, *ofp;
	int	control;
	char in_type=1, in_end_line_type=0, out_type=1, out_end_line_type=0;

	if ( argc > 6 && !strcasecmp(argv[5],argv[6]) ) {
		printf( FILE_ERR );
		exit(0);
	}

	// input file
	if (argv[5] != NULL && argc > 5){
		ifp = fopen(argv[5], "r");
		if (ifp == NULL) {
			printf( IN_FILE_ERR , argv[5]);
			exit(0);
		}
	} else {
		ifp = stdin;
	}
	
	// output file
	if (argv[6] != NULL && argc > 6){
		ofp = fopen(argv[6], "wb");
		if (ofp == NULL) {
			printf( OUT_FILE_ERR, argv[6]);
			exit(0);
		}
	} else {
		ofp = stdout;
	}

	// control arg
	if ( argv[1]==NULL || argv[2]==NULL || argv[3]==NULL || argv[4]==NULL) {
		print_help();
		exit(0);
	}


	// convert text arg to num args for convert_in function
	     if ( !strcasecmp(argv[1],"utf7") )	in_type = UTF7;
	else if ( !strcasecmp(argv[1],"utf8") )	in_type = UTF8;
	else if ( !strcasecmp(argv[1],"utf16") )	in_type = UTF16;
	else if ( !strcasecmp(argv[1],"utf16be") )	in_type = UTF16be;
	else if ( !strcasecmp(argv[1],"utf16le") )	in_type = UTF16le;
	
	     if ( !strcasecmp(argv[2],"-") )	in_end_line_type = NOT;
	else if ( !strcasecmp(argv[2],"lf") )	in_end_line_type = LF;
	else if ( !strcasecmp(argv[2],"cr") )	in_end_line_type = CR;
	else if ( !strcasecmp(argv[2],"crlf") )	in_end_line_type = CRLF;
	
	     if ( !strcasecmp(argv[3],"utf7") )	out_type = UTF7;
	else if ( !strcasecmp(argv[3],"utf8") )	out_type = UTF8;
	else if ( !strcasecmp(argv[3],"utf16be") )	out_type = UTF16be;
	else if ( !strcasecmp(argv[3],"utf16le") )	out_type = UTF16le;
	
	     if ( !strcasecmp(argv[4],"-") )	out_end_line_type = NOT;
	else if ( !strcasecmp(argv[4],"lf") )	out_end_line_type = LF;
	else if ( !strcasecmp(argv[4],"cr") )	out_end_line_type = CR;
	else if ( !strcasecmp(argv[4],"crlf") )	out_end_line_type = CRLF;

	// convert file
	control = convert_in(in_type, in_end_line_type, ifp, out_type, out_end_line_type, ofp);
	
	// return code control
	if (control == 1) {
		print_help();
		exit(0);
	} else if (control == 2) {
		printf( UTF16_NO_DETECT_ERR );
		exit(0);
	} else if (control == 3) {
		printf( END_LINE_ERR );
		exit(0);
	}

	// end program ;-)
	fclose(ifp);
	fclose(ofp);
	return 0;
}